@@ -1,10 +1,20 @@
-from . import Stage, Agent, __USER_TEXT__, MultiAgent, StageType
+from .__stage__ import Stage, __USER_TEXT__, StageType
+from .__agent__ import Agent, MultiAgent
+from .__tool__ import get_value_from_dict_by_multi_name
 from .jieba_zh import analyse
 import re
 
 __RE_STAGE__ = "__RE_STAGE__"
 __QA_STAGE__ = "__QA_STAGE__"
 __SWITCH_STAGE__ = StageType.SWITCH
+__LIB_SWITCH_STAGE__ = "__LIB_SWITCH_STAGE__"
+
+#
+__QUESTIONS_LABEL__ = ["question", "says"]
+__WELCOME_SAYING_LABELS__ = ["sys_reply_q1", "sys_welcome"]
+__REFUSE_SAYING_LABELS__ = ["sys_reply_q2", "sys_refuse"]
+__COMPLETE_SAYING_LABELS__ = ["sys_reply_complete", "sys_complete"]
+__DISSABLE_WELCOME_LABEL__ = ["__DISSABLE_Q1__", "DISSABLE_WELCOME"]
 
 from .nlp_tool import similar
 
@@ -32,15 +42,20 @@ class QAStage(Stage):
         corpus = data.get(self.__CORPUS__)
         for k, v in corpus.items():
             self.corpus[k.replace(" ", "")] = v.replace(" ", "")
-        self.sys_reply_q1 = data.get(self.__QUESTIONS__).get("sys_reply_q1")
-        self.sys_reply_q2 = data.get(self.__QUESTIONS__).get("sys_reply_q2")
-        self.sys_reply_complete = data.get(self.__QUESTIONS__).get("sys_reply_complete")
-        self.saved_name:dict = data.get(self.__SAVED_NAME__,{})
-        self.qa_threshold = data.get(self.__QA_THRESHOLD__, 0.5)
+
+        #
+        self.questions = get_value_from_dict_by_multi_name(d=data, names=__QUESTIONS_LABEL__)
+        self.sys_reply_q1 = get_value_from_dict_by_multi_name(d=self.questions, names=__WELCOME_SAYING_LABELS__)
+        self.sys_reply_q2 = get_value_from_dict_by_multi_name(d=self.questions, names=__REFUSE_SAYING_LABELS__)
+        self.sys_reply_complete = get_value_from_dict_by_multi_name(d=self.questions,
+                                                                    names=__COMPLETE_SAYING_LABELS__)
+        self.saved_name: dict = data.get(self.__SAVED_NAME__, {})
+        self.qa_threshold = data.get(self.__QA_THRESHOLD__, 0.1)
 
         # TODO: RENAME `refactor_questions` -> `refactor_corpus_questions`
         self.refactor_questions = data.get(self.__REFACTOR_QUESTION__, False)
-        if data.get(self.__DISSABLE_Q1__, False):
+
+        if get_value_from_dict_by_multi_name(d=data, names=__DISSABLE_WELCOME_LABEL__, default=False):
             self.is_first_access = self.disabel_is_first_access
 
     def __request_similar_api__(self, text, corpus):
@@ -49,9 +64,9 @@ class QAStage(Stage):
     def __encode_corpus__(self):
         source_corpus = list(self.corpus.keys())
         dict_corpus = {self.__keyword__(self.__stop_word__(s)): s for s in source_corpus}
-        return list(self.corpus.keys() if not self.refactor_questions else dict_corpus .keys()),dict_corpus
+        return list(self.corpus.keys() if not self.refactor_questions else dict_corpus.keys()), dict_corpus
 
-    def __decode_corpus__(self, worker_response,new_dict_corpus):
+    def __decode_corpus__(self, worker_response, new_dict_corpus):
         best_res_content = str(worker_response[0][0]).replace(" ", "")
         if self.refactor_questions:
             best_res_content = new_dict_corpus[best_res_content] if best_res_content in new_dict_corpus else '0'
@@ -59,12 +74,12 @@ class QAStage(Stage):
 
     def is_fit_needs_n_gen_entity(self, kwargs) -> (bool, dict):
         user_text = kwargs.get(__USER_TEXT__, "")
-        pass_token = True
+
         corpus, new_dict_corpus = self.__encode_corpus__()
 
         responds = self.__request_similar_api__(user_text, corpus)
         worker_response = responds["worker response"]["ans"]
-        best_res_content = self.__decode_corpus__(worker_response,new_dict_corpus)
+        best_res_content = self.__decode_corpus__(worker_response, new_dict_corpus)
 
         ##
         best_res_score = worker_response[0][1]
@@ -83,13 +98,16 @@ class QAStage(Stage):
         kwargs = self.set_default_var(kwargs, __QA_RESPOND_QUESTION__, best_res_content)
         kwargs = self.set_default_var(kwargs, __QA_RESPOND_SCORE__, best_res_score)
         if best_res_score >= self.qa_threshold:
+            pass_token = True
             kwargs = self.set_default_var(kwargs, __QA_RESPOND_THRESHOLD__, True)
         else:
+            pass_token = False
             kwargs = self.set_default_var(kwargs, __QA_RESPOND_THRESHOLD__, False)
         return pass_token, kwargs
 
     @staticmethod
-    def disabel_is_first_access(kwargs, stage_id): return False
+    def disabel_is_first_access(kwargs, stage_id):
+        return False
 
     @staticmethod
     def __stop_word__(text, pronouns=True, common_skip=True, road_name=True, symbol=True):
@@ -104,30 +122,19 @@ class QAStage(Stage):
         if symbol:
             filter_set += ["-", "，", "？", "。"]
         for stopword in filter_set:
-            text = text.replace(stopword," ")
+            text = text.replace(stopword, " ")
         return text
 
-        # res = requests.post(url=f"http://52.175.156.240:8000/cts/sentence_stop_words", json={
-        #     "sentence": text
-        # })
-        # return res.json()["refactor_sentence"]
-
     @staticmethod
-    def __keyword__(text,k=10):
+    def __keyword__(text, k=10):
         allowed_words = ""
         list_en = [i for i in "abcdefghijklmnopqrstuvwxyz"] + [i for i in "abcdefghijklmnopqrstuvwxyz".upper()]
-        list_math = [str(i) for i in range(0,10)]
-        list_bus = ["紅","棕","綠", "橘", "藍","黃","區","副"]
+        list_math = [str(i) for i in range(0, 10)]
+        list_bus = ["紅", "棕", "綠", "橘", "藍", "黃", "區", "副"]
         new_text = analyse.tfidf(text, topK=k, withWeight=True)
-        allowed_words += "".join([t for t, v in new_text]+list_bus+list_en+list_math)
+        allowed_words += "".join([t for t, v in new_text] + list_bus + list_en + list_math)
         return_text = "".join([w for w in text if w in allowed_words])
         return return_text
-        # res = requests.post(url=f"http://52.175.156.240:8000/cts/sentence_stop_words", json={
-        #     "sentence": text,
-        #     "k": k
-        # })
-
-        # return res.json()["refactor_sentence"]
 
 
 class REStage(Stage):
@@ -137,10 +144,14 @@ class REStage(Stage):
         super(REStage, self).__init__()
         self.raw_data = data
         self.stage_type = data.get("stage_type")
-        self.sys_reply_q1 = data.get("question").get("sys_reply_q1")
-        self.sys_reply_q2 = data.get("question").get("sys_reply_q2")
-        self.sys_reply_complete = data.get("question").get("sys_reply_complete")
-        self.is_fits = data.get("is_fits")
+
+        #
+        self.questions = get_value_from_dict_by_multi_name(d=data, names=__QUESTIONS_LABEL__)
+        self.sys_reply_q1 = get_value_from_dict_by_multi_name(d=self.questions, names=__WELCOME_SAYING_LABELS__)
+        self.sys_reply_q2 = get_value_from_dict_by_multi_name(d=self.questions, names=__REFUSE_SAYING_LABELS__)
+        self.sys_reply_complete = get_value_from_dict_by_multi_name(d=self.questions,
+                                                                    names=__COMPLETE_SAYING_LABELS__)
+        self.is_fits = data.get("is_fits", [])
         if data.get("__DISSABLE_Q1__", False):
             self.is_first_access = self.disabel_is_first_access
 
@@ -186,7 +197,6 @@ class LibSwitchStage(Stage):
         self.stage_type = StageType.SWITCH
         self.stages_filter = data.get("stages_filter")
 
-
     @staticmethod
     def is_first_access(data, stage_id):
         return False
@@ -195,25 +205,32 @@ class LibSwitchStage(Stage):
 
         for filter_tuple in self.stages_filter:
             label, text, stages_label = filter_tuple
-            try:
-                if self.get_default_var(kwargs, label) == text:
-                    return stages_label
-            except KeyError as e:
-                RuntimeError("UnDefined Path")
+            #
+            if label == "*" and text is True:
+                return stages_label
+
+            #
+            assert isinstance(label, type(text)), "切換路線之條件與限定結果的類型不一致，必須保持兩者都是`str` 或是兩者都是`list`"
+            if isinstance(label, str):
+                label, text = [label], [text]
+
+            assert len(label) == len(text), "切換路線之條件與限定結果的數量不一致"
+
+            #
+
+            pass_token = True
+            for l, t in zip(label, text):
+                if self.get_default_var(kwargs, l) != t:
+                    pass_token = False
+            if pass_token:
+                return stages_label
+
         raise RuntimeError("Not Found Path")
 
     def is_fit_needs_n_gen_entity(self, kwargs) -> (bool, dict):
         raise RuntimeError
 
 
-def gen_agent(stage_jsons:[str]):
-    stages = []
-    for stage_json in stage_jsons:
-        mock_class = __LIB_STAGES__[stage_json["stage_type"]]
-        stages.append(mock_class(stage_json))
-    return Agent(stages)
-
-
 def gen_multi_agent(stage_dict: dict):
     stages = {}
     for stages_label, stage_jsons in stage_dict.items():
@@ -225,12 +242,14 @@ def gen_multi_agent(stage_dict: dict):
     return MultiAgent(stages)
 
 
+def gen_agent(stage_dict: dict):
+    return gen_multi_agent(stage_dict)
+
 
 #
 __LIB_STAGES__ = {
     __RE_STAGE__: REStage,
     __QA_STAGE__: QAStage,
-    __SWITCH_STAGE__: LibSwitchStage
+    __SWITCH_STAGE__: LibSwitchStage,
+    __LIB_SWITCH_STAGE__: LibSwitchStage
 }
-
-
